//declare TimeZone=tz and DST, else autoset timezone will run.
//#declare DST = 1;
//#declare TimeZone = -8;

#macro julian(Y,M,D)
  #if (M < 3)
    #local year = Y - 1;
    #local m = M + 12;
  #else #local m = M; #local year = Y; #end
  #local A = floor(year / 100.0);
  #local B = 2 - A + floor(A / 4.0);
  #local J = floor(365.25 * (year + 4716.0)) + floor(30.6001 * (m + 1)) + D + B - 1524.5;
  #local H = floor(val(date("%H")) - (TimeZone+DST));
  #local J = J + ((H*3600.0 + val(date("%M"))*60.0 + val(date("%S")))/86400.0);
  J
#end
#macro gmt2julian(Y,M,D,h,n,s)
  #if (M < 3)
    #local year = Y - 1;
    #local m = M + 12;
  #else #local m = M; #local year = Y; #end
  #local A = floor(year / 100.0);
  #local B = 2 - A + floor(A / 4.0);
  #local J = floor(365.25 * (year + 4716.0)) + floor(30.6001 * (m + 1)) + D + B - 1524.5;
  #local J = J + ((h*3600.0 + n*60.0 + s)/86400.0);
  J
#end
#macro local2julian(Y,M,D,h,n,s)
  #if (M < 3)
    #local year = Y - 1;
    #local m = M + 12;
  #else #local m = M; #local year = Y; #end
  #local A = floor(year / 100.0);
  #local B = 2 - A + floor(A / 4.0);
  #local J = floor(365.25 * (year + 4716.0)) + floor(30.6001 * (m + 1)) + D + B - 1524.5;
  #local J = J + ((floor(h - (TimeZone+DST))*3600.0 + n*60.0 + s)/86400.0);
  J
#end
#macro j2ctime(j)
  #local c = j - 2440587.5;
  #local c = c * 86400.0;
  round(c,0)
#end

#declare month=array[13] { "Dec","Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec" }

#macro j2datetime(j)
  #local d = array[3] { 0,0,0 }
  #local d = jdate(j);
  #declare Date = concat(month[_clamp(d[1],1,12)]," ",str(d[2],0,0)," ",str(d[0],0,0))
  #declare Time = jhms(j)
  #declare GMT = 1;
#end 

#macro jdate(jd)
  #local j=jd+0.5;
  #local _z = floor(j);
  #local f = j - _z;
  #if (_z < 2299161.0) #local a = _z;
  #else
    #local alph = floor((_z - 1867216.25) / 36524.25);
    #local a = (_z + 1 + alph - floor(alph / 4));
  #end
  #local b = a + 1524;
  #local c = floor((b - 122.1) / 365.25);
  #local d = floor(365.25 * c);
  #local e = floor((b - d) / 30.6001);
  #if (e < 14) #local m = e - 1;
  #else #local m = e - 13; #end
  #if (m > 2) #local _y = c - 4716;
  #else #local _y = c - 4715; #end
  #local dt=array[3]
  #local dt[0] = floor(_y);
  #local dt[1] = floor(m);
  #local dt[2] = floor(b - d - floor(30.6001 * e) + f);
  dt
#end
#macro jhms(jd)
  #local j = jd+0.5;
  #local j = j - floor(j);
  #local b = j * 86400.0;
  #local h = floor(b / 3600);
  #local m = floor(mod(b / 60, 60));
  #local s = floor(mod(b, 60));
  #if (h > 23) #local h=h-24; #end
  concat(pad(h,2,"0",0),":",pad(m,2,"0",0),":",pad(s,2,"0",0))
#end
#macro pad(n,l,c,d) //pad(number, to len, with char, .decimal len) return string
  #local s = str(n,0,d)
  #while (strlen(s)<l)
    #local s = concat(c,s)
  #end
  s
#end
/////////////////////////////////////////////////////////////// AutoDate  MegaPov
//#version unofficial MegaPov 1.21;
#version 3.63;
#ifdef (TimeZone)
  //#declare Time = date("%H:%M:%S")
  //#declare Date = date("%B %d %Y")
  ////////////////////////////////////////////////////////////// TimeZone
  //#declare DST = 1;
  //#declare TimeZone = -8;
  #declare JulianDate = julian(val(date("%Y")),val(date("%m")),val(date("%d")));
  #declare ctime = j2ctime(JulianDate);
  j2datetime(JulianDate)
  echof(JulianDate)
#end

/*
Table 2.1. The following time formatting strings are available:

a  Abbreviated weekday name. 
A Full weekday name. 
b Abbreviated month name. 
B Full month name. 
c The strftime() format equaling the format string of "%x %X". 
d Day of the month as a decimal number. 
H The hour (24-hour clock) as a decimal number from 00 to 23. 
I The hour (12-hour clock) as a decimal number from 01 to 12 
j The day of the year as a decimal number from 001 to 366 
m The month as a decimal number from 01 to 12. 
M The minute as a decimal number from 00 to 59. 
p "AM" or "PM". 
S The seconds as a decimal number from 00 to 59. 
U The week number of the year as a decimal number from 00 to 52. Sunday is considered the first day of the week. 
w The weekday as a decimal number from 0 to 6. Sunday is (0) zero. 
W The week of the year as a decimal number from 00 to 51. Monday is the first day of the week. 
x The date representation of the current locale. 
X The time representation of the current locale. 
y The last two digits of the year as a decimal number. 
Y The century as a decimal number. 
z The time zone name or nothing if it is unknown. 
%   The percent sign is displayed. 

(GMT) Casablanca, Monrovia
(GMT) Greenwich Mean Time : Dublin, Edinburgh, Lisbon, London
(GMT+01:00) Amsterdam, Berlin, Bern, Rome, Stockholm, Vienna
(GMT+01:00) Belgrade, Bratislava, Budapest, Ljubljana, Prague
(GMT+01:00) Brussels, Copenhagen, Madrid, Paris
(GMT+01:00) Sarajevo, Skopje, Warsaw, Zagreb
(GMT+01:00) West Central Africa
(GMT+02:00) Athens, Beirut, Istanbul, Minsk
(GMT+02:00) Bucharest
(GMT+02:00) Cairo
(GMT+02:00) Harare, Pretoria
(GMT+02:00) Helsinki, Kyiv, Riga, Sofia, Tallinn, Vilnius
(GMT+02:00) Jerusalem
(GMT+03:00) Baghdad
(GMT+03:00) Kuwait, Riyadh
(GMT+03:00) Moscow, St. Petersburg, Volgograd
(GMT+03:00) Nairobi
(GMT+03:30) Tehran
(GMT+04:00) Abu Dhabi, Muscat
(GMT+04:00) Baku, Tbilisi, Yerevan
(GMT+04:30) Kabul
(GMT+05:00) Ekaterinburg
(GMT+05:00) Islamabad, Karachi, Tashkent
(GMT+05:30) Chennai, Kolkata, Mumbai, New Delhi
(GMT+05:45) Kathmandu
(GMT+06:00) Almaty, Novosibirsk
(GMT+06:00) Astana, Dhaka
(GMT+06:00) Sri Jayawardenepura
(GMT+06:30) Rangoon
(GMT+07:00) Bangkok, Hanoi, Jakarta
(GMT+07:00) Krasnoyarsk
(GMT+08:00) Beijing, Chongqing, Hong Kong, Urumqi
(GMT+08:00) Irkutsk, Ulaan Bataar
(GMT+08:00) Kuala Lumpur, Singapore
(GMT+08:00) Perth
(GMT+08:00) Taipei
(GMT+09:00) Osaka, Sapporo, Tokyo
(GMT+09:00) Seoul
(GMT+09:00) Yakutsk
(GMT+09:30) Adelaide
(GMT+09:30) Darwin
(GMT+10:00) Brisbane
(GMT+10:00) Canberra, Melbourne, Sydney
(GMT+10:00) Guam, Port Moresby
(GMT+10:00) Hobart
(GMT+10:00) Vladivostok
(GMT+11:00) Magadan, Solomon Is., New Caledonia
(GMT+12:00) Auckland, Wellington
(GMT+12:00) Fiji, Kamchatka, Marshall Is.
(GMT+13:00) Nuku'alofa
(GMT-01:00) Azores
(GMT-01:00) Cape Verde Is.
(GMT-02:00) Mid-Atlantic
(GMT-03:00) Brasilia
(GMT-03:00) Buenos Aires, Georgetown
(GMT-03:00) Greenland
(GMT-03:30) Newfoundland
(GMT-04:00) Atlantic Time (Canada)
(GMT-04:00) Caracas, La Paz
(GMT-04:00) Santiago
(GMT-05:00) Bogota, Lima, Quito
(GMT-05:00) Eastern Time (US & Canada)
(GMT-05:00) Indiana (East)
(GMT-06:00) Central America
(GMT-06:00) Central Time (US & Canada)
(GMT-06:00) Guadalajara, Mexico City, Monterrey
(GMT-06:00) Saskatchewan
(GMT-07:00) Arizona
(GMT-07:00) Chihuahua, La Paz, Mazatlan
(GMT-07:00) Mountain Time (US & Canada)
(GMT-08:00) Pacific Time (US & Canada); Tijuana
(GMT-09:00) Alaska
(GMT-10:00) Hawaii
(GMT-11:00) Midway Island, Samoa
(GMT-12:00) International Date Line West
*/
#declare tzones = array[75][7] {
{ "0","Casablanca","Monrovia",".",".",".","." }
{ "0","Greenwich Mean Time : Dublin","Edinburgh","Lisbon","London",".","." }
{ "1","Amsterdam","Berlin","Bern","Rome","Stockholm","Vienna" }
{ "1","Belgrade","Bratislava","Budapest","Ljubljana","Prague","." }
{ "1","Brussels","Copenhagen","Madrid","Paris",".","." }
{ "1","Sarajevo","Skopje","Warsaw","Zagreb",".","." }
{ "1","West Central Africa",".",".",".",".","." }
{ "2","Athens","Beirut","Istanbul","Minsk",".","." }
{ "2","Bucharest",".",".",".",".","." }
{ "2","Cairo",".",".",".",".","." }
{ "2","Harare","Pretoria",".",".",".","." }
{ "2","Helsinki","Kyiv","Riga","Sofia","Tallinn","Vilnius" }
{ "2","Jerusalem",".",".",".",".","." }
{ "3","Baghdad",".",".",".",".","." }
{ "3","Kuwait","Riyadh",".",".",".","." }
{ "3","Moscow","St. Petersburg","Volgograd",".",".","." }
{ "3","Nairobi",".",".",".",".","." }
{ "3.5","Tehran",".",".",".",".","." }
{ "4","Abu Dhabi","Muscat",".",".",".","." }
{ "4","Baku","Tbilisi","Yerevan",".",".","." }
{ "4.5","Kabul",".",".",".",".","." }
{ "5","Ekaterinburg",".",".",".",".","." }
{ "5","Islamabad","Karachi","Tashkent",".",".","." }
{ "5.5","Chennai","Kolkata","Mumbai","New Delhi",".","." }
{ "5.75","Kathmandu",".",".",".",".","." }
{ "6","Almaty","Novosibirsk",".",".",".","." }
{ "6","Astana","Dhaka",".",".",".","." }
{ "6","Sri Jayawardenepura",".",".",".",".","." }
{ "6.5","Rangoon",".",".",".",".","." }
{ "7","Bangkok","Hanoi","Jakarta",".",".","." }
{ "7","Krasnoyarsk",".",".",".",".","." }
{ "8","Beijing","Chongqing","Hong Kong","Urumqi",".","." }
{ "8","Irkutsk","Ulaan Bataar",".",".",".","." }
{ "8","Kuala Lumpur","Singapore",".",".",".","." }
{ "8","Perth",".",".",".",".","." }
{ "8","Taipei",".",".",".",".","." }
{ "9","Osaka","Sapporo","Tokyo",".",".","." }
{ "9","Seoul",".",".",".",".","." }
{ "9","Yakutsk",".",".",".",".","." }
{ "9.5","Adelaide",".",".",".",".","." }
{ "9.5","Darwin",".",".",".",".","." }
{ "10","Brisbane",".",".",".",".","." }
{ "10","Canberra","Melbourne","Sydney",".",".","." }
{ "10","Guam","Port Moresby",".",".",".","." }
{ "10","Hobart",".",".",".",".","." }
{ "10","Vladivostok",".",".",".",".","." }
{ "11","Magadan","Solomon Is.","New Caledonia",".",".","." }
{ "12","Auckland","Wellington",".",".",".","." }
{ "12","Fiji","Kamchatka","Marshall Is.",".",".","." }
{ "13","Nuku'alofa",".",".",".",".","." }
{ "-1","Azores",".",".",".",".","." }
{ "-1","Cape Verde Is.",".",".",".",".","." }
{ "-2","Mid-Atlantic",".",".",".",".","." }
{ "-3","Brasilia",".",".",".",".","." }
{ "-3","Buenos Aires","Georgetown",".",".",".","." }
{ "-3","Greenland",".",".",".",".","." }
{ "-3.5","Newfoundland",".",".",".",".","." }
{ "-4","Atlantic Time (Canada)",".",".",".",".","." }
{ "-4","Caracas","La Paz",".",".",".","." }
{ "-4","Santiago",".",".",".",".","." }
{ "-5","Bogota","Lima","Quito",".",".","." }
{ "-5","Eastern Time (US & Canada)",".",".",".",".","." }
{ "-5","Indiana (East)",".",".",".",".","." }
{ "-6","Central America",".",".",".",".","." }
{ "-6","Central Time (US & Canada)",".",".",".",".","." }
{ "-6","Guadalajara","Mexico City","Monterrey",".",".","." }
{ "-6","Saskatchewan",".",".",".",".","." }
{ "-7","Arizona",".",".",".",".","." }
{ "-7","Chihuahua","La Paz","Mazatlan",".",".","." }
{ "-7","Mountain Time (US & Canada)",".",".",".",".","." }
{ "-8","Pacific Time (US & Canada); Tijuana",".",".",".",".","." }
{ "-9","Alaska",".",".",".",".","." }
{ "-10","Hawaii",".",".",".",".","." }
{ "-11","Midway Island","Samoa",".",".",".","." }
{ "-12","International Date Line West",".",".",".",".","." }
}



#macro findword(sst,st)
  #local i=0; #local j=0; #local result = 0;
  #while (j<strlen(st))

    #if (j+1=strlen(st))
       #if (!strcmp(sst,substr(st,i+1,j+1-i))) #local result = 1; #end
    #else #if (!strcmp(" ",substr(st,j+1,1)))
    
     #if (!strcmp(sst,substr(st,i+1,j-i))) #local result = 1; #end
        #local j=j+1;
        #local i = j;
       
    #end  #end
    #local j=j+1;

#end
    result
#end

#macro remove_token(st,sst,c)
  #local i=0; #local j=0; #local newstr = " "
  #while (j<strlen(st))

    #if (j+1=strlen(st))
       #if (strcmp(sst,substr(st,i+1,j+1-i)))
       #local newstr = concat(newstr,substr(st,i+1,j+1-i)," ") #end
    #else #if (!strcmp(c,substr(st,j+1,1)))
    
        #if (strcmp(sst,substr(st,i+1,j-i))) 
        #local newstr = concat(newstr,substr(st,i+1,j-i)," ") #end
        #local j=j+1;
        #local i = j;
       
    #end  #end
    #local j=j+1;

 #end
  #local newstr = substr(newstr,2,strlen(newstr)-2)
    newstr
#end 


#macro searchtz(st)
  #local s = remove_token(st,"Daylight"," ")
  #local s = remove_token(s,"Standard"," ")
  #local i=0; #local found = 255;
  #while (i < dimension_size(tzones, 1))
    #local j=0;
    #while (j < dimension_size(tzones, 2))
      #local s2 = tzones[i][j]
      #if (!strcmp(".",s2)) #break #end
      #if (!strcmp(substr(s,1,_clamp(strlen(s),1,strlen(s2))),
        substr(s2,1,_clamp(strlen(s),1,strlen(s2)))))
        #local found = val(tzones[i][0]); #break
      #end
      #local j=j+1;
    #end
    #if (found != 255) #break #end
    #local i=i+1;
  #end
  #if (found = 255) #local found = 0; echo("Warning: TimeZone Was Not Found. See megatime.inc") cr() #end
  found
#end
//#declare secday = 86400.0; 
#macro setlocaltime(j)
     j2datetime((TimeZone+DST)*3600.0/86400.0+j)
     #declare GMT = 0;
#end
////////////////////////////// Set TimeZone \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

#macro TZset(sw)
   #local tzstr = date("%z")
   cr() echo(concat("TZ: ",tzstr)) cr()
   
   #declare DST = findword("Daylight",tzstr);
   #declare TimeZone = searchtz(tzstr);
   #declare JulianDate = julian(val(date("%Y")),val(date("%m")),val(date("%d")));
   #declare ctime = j2ctime(JulianDate);   echof(JulianDate) cr()
   
   setlocaltime(JulianDate)// local date
   echo("Local: ") echo(Date) echo(" ") echo(Time) echo(" DST ") echoi(DST) cr()
   
   j2datetime(JulianDate) //GMT date
   echo("GMT: ") echo(Date) echo(" ") echo(Time) echo(" DST ") echoi(0) cr()
   #if (sw) setlocaltime(JulianDate) #end
  
   //#declare lTime = date("%H:%M:%S")
   //#declare lDate = date("%B %d %Y")
#end   
#ifndef (TimeZone)
  TZset(10)   // 0 set GMT; 1 set local
#end
//echof(local2julian(2009,7,12,10,56,55)) cr()
//echof(gmt2julian(2009,7,12,15,56,55)) cr()